<?php
/**
 * Handles integration with the Delivery Slots plugin plugin.
 *
 * @package   Barn2/woocommerce-lead-time
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */

namespace Barn2\Plugin\WC_Lead_Time\Integration;

use Barn2\Plugin\WC_Lead_Time\Display;
use Barn2\Plugin\WC_Lead_Time\Util as WC_Lead_TimeUtil;
use Barn2\Plugin\WC_Lead_Time\Dependencies\Lib\Registerable,
	Barn2\Plugin\WC_Lead_Time\Dependencies\Lib\Util;

/**
 * Provides integration of WLT with the Delivery slots plugin by Iconic.
 *
 * While the Iconic's plugin already provides integration with WLT,
 * their current integration only supports static lead times.
 *
 * Because version 2.0+ of WLT adds support for dynamic lead times,
 * we need to disable their filter 'iconic_wds_min_delivery_date',
 * and attach our new custom logic.
 *
 * Majority of the code is borrowed from Iconic's integration, the code
 * has been adapted to support the new metadata and options.
 */
class Delivery_Slots implements Registerable {

	public function register() {
		add_filter( 'iconic_wds_min_delivery_date', [ $this, 'set_min_delivery_date' ], 100 );
	}

	public function set_min_delivery_date( $data = array() ) {

		if ( ! function_exists( 'Barn2\Plugin\WC_Lead_Time\wlt' ) ) {
			return $data;
		}

		$unit = self::get_lead_time_units();
		$lead_time = self::get_longest_lead_time_date();

		if ( empty( $lead_time ) || isset( $lead_time['time'] ) && empty( $lead_time['time'] ) ) {
			return $data;
		}

		$format = $lead_time['format'];
		$lead_time = is_array( $lead_time ) ? $lead_time['time'] : $lead_time;

		if ( $format === 'dynamic' ) {

			if ( ! WC_Lead_TimeUtil::should_date_display( $lead_time ) ) {
				return $data;
			} else {
				$today = ( new \DateTime() )
					->setTimezone( wp_timezone() )
					->setTime( 0, 0, 0 );

				$target_date = ( new \DateTime() )
					->setTimezone( wp_timezone() )
					->setTime( 0, 0, 0 )
					->setTimestamp( $lead_time );

				$interval = $today->diff( $target_date );

				return [
					'days_to_add' => $interval->d,
					'timestamp'   => $lead_time,
				];
			}
		}

		return array(
			'days_to_add' => 'days' === $unit ? $lead_time : $lead_time * 7,
			'timestamp'   => strtotime( '+' . $lead_time . ' ' . $unit, time() ),
		);

	}

	public static function get_longest_lead_time_date() {

		$cart_items = WC()->cart->get_cart();

		if ( empty( $cart_items ) ) {
			return false;
		}

		$lead_time = 0;

		foreach ( $cart_items as $cart_item ) {
			$product_lead_time = self::get_product_lead_time( $cart_item['data'] );
			$check = isset( $product_lead_time['time'] ) ? absint( $product_lead_time['time'] ) : $product_lead_time;

			if ( $check && $check <= $lead_time ) {
				continue;
			}

			$lead_time = $product_lead_time;
		}

		return $lead_time;

	}

	/**
	 * Get lead time units.
	 *
	 * @return mixed|string|void
	 */
	public static function get_lead_time_units() {
		$unit = get_option( 'wclt_units', 'default' );

		return 'weeks' !== $unit ? 'days' : $unit;
	}

	/**
	 * Get product lead time.
	 *
	 * @param WC_Product $product Product.
	 *
	 * @return bool|int
	 */
	public static function get_product_lead_time( $product ) {
		$format = Display::get_lead_time_format_raw( $product );
		$lead_time = Display::get_lead_time_raw( $product );

		if ( is_array( $lead_time ) && isset( $lead_time['dynamic'] ) && $lead_time['dynamic'] === true ) {
			$format = 'dynamic';
			$lead_time = $lead_time['lead_time'];
		}

		return [
			'format' => $format,
			'time' => $lead_time,
		];
	}

}
